#include <unistd.h>
#include <sys/time.h>
#include <signal.h>
#include "sml.h"
#include "strerr.h"
#include "timeout.h"

static void
sigalrm_handler(int signo)
{
    int nwritten, rc;

    for (nwritten = 0; nwritten < safe_output->len; nwritten += rc) {
	rc = write(1, safe_output->s + nwritten, safe_output->len - nwritten);
	if (rc < 0)
	    strerr_die1sys(1, "write: ");
    }

    _exit(0);
}

static void
sigsegv_handler(int signo)
{
/*    strerr_warn1("sigsegv!", 0);*/
    timeout();
}

void
set_timeout(long seconds)
{
    struct sigaction sa;
    struct itimerval it;

    sa.sa_handler = sigalrm_handler;
    sigemptyset(&sa.sa_mask);
    sa.sa_flags = 0;
    if (sigaction(SIGALRM, &sa, 0) < 0)
	strerr_die1sys(1, "sigaction(SIGALRM): ");

    /* Also capture SIGSEGV in case of fatal bug. */
    sa.sa_handler = sigsegv_handler;
    if (sigaction(SIGSEGV, &sa, 0) < 0)
	strerr_die1sys(1, "sigaction(SIGSEGV): ");

    it.it_interval.tv_sec = 0;
    it.it_interval.tv_usec = 0;
    it.it_value.tv_sec = seconds;
    it.it_value.tv_usec = 0;
    if (setitimer(ITIMER_REAL, &it, 0) < 0)
	strerr_die1sys(1, "setitimer: ");
}

void
timeout(void)
{
    kill(getpid(), SIGALRM);
}

void
quit(void)
{
/*    strerr_warn1("quitting", 0);*/
    timeout();
}

